@@ -8,16 +8,10 @@
 #'   values.
 #' @export
 #'
+#' @inheritParams genetic_algorithm
 #' @param fit A list of all fitness-evaluated individuals
 #' @param Grid Is the indexed grid output from \code{\link{grid_area}}
 #' @param teil A numeric value that determines the selection percentage
-#' @param elitism Boolean value which indicates whether elitism should be
-#'   included or not.
-#' @param nelit If \code{elitism} is TRUE, then this input variable determines
-#'   the amount of individuals in the elite group.
-#' @param selstate Determines which selection method is used, "FIX" selects a
-#'   constant percentage and "VAR" selects a variable percentage, depending on
-#'   the development of the fitness values.
 #' @param verbose If TRUE, will print out further information.
 #'
 #' @family Genetic Algorithm Functions
@@ -25,16 +19,17 @@
 #'   which shows all selected individuals. Element 2 represent the mean fitness
 #'   values of each parental team.
 #' @examples \dontrun{
-#' ## Create a random rectangular shapefile
-#' library(sp)
-#' Polygon1 <- Polygon(rbind(c(0, 0), c(0, 2000), c(2000, 2000), c(2000, 0)))
-#' Polygon1 <- Polygons(list(Polygon1),1);
-#' Polygon1 <- SpatialPolygons(list(Polygon1))
-#' Projection <- "+init=epsg:3035"
-#' proj4string(Polygon1) <- CRS(Projection)
+#' ## Exemplary input Polygon with 2km x 2km:
+#' library(sf)
+#' Polygon1 <- sf::st_as_sf(sf::st_sfc(
+#'   sf::st_polygon(list(cbind(
+#'     c(4498482, 4498482, 4499991, 4499991, 4498482),
+#'     c(2668272, 2669343, 2669343, 2668272, 2668272)))), 
+#'   crs = 3035
+#' ))
 #'
 #' ## Calculate a Grid and an indexed data.frame with coordinates and grid cell Ids.
-#' Grid1 <- grid_area(shape = Polygon1,resol = 200,prop = 1);
+#' Grid1 <- grid_area(shape = Polygon1, size = 200, prop = 1);
 #' Grid <- Grid1[[1]]
 #' AmountGrids <- nrow(Grid)
 #'
@@ -42,7 +37,7 @@
 #' wind <- as.data.frame(cbind(ws=12,wd=0))
 #' wind <- list(wind, probab = 100)
 #' fit <- fitness(selection = startsel, referenceHeight = 100, RotorHeight=100,
-#'                SurfaceRoughness=0.3,Polygon = Polygon1, resol1 = 200,
+#'                SurfaceRoughness=0.3, Polygon = Polygon1, resol1 = 200,
 #'                rot = 20, dirspeed = wind, 
 #'                srtm_crop = "", topograp = FALSE, cclRaster = "")
 #' allparks <- do.call("rbind",fit);
@@ -54,8 +49,7 @@
 #' selec6best <- selection(fit, Grid, 2, TRUE, 6, "FIX")
 #' selec6best <- selection(fit, Grid, 4, FALSE, 6, "FIX")
 #' }
-selection         <- function(fit, Grid, teil, elitism, nelit, 
-                               selstate, verbose) {
+selection <- function(fit, Grid, teil, elitism, nelit, selstate, verbose) {
   if (missing(verbose)) {
     verbose <- FALSE
   }
@@ -113,7 +107,8 @@ selection         <- function(fit, Grid, teil, elitism, nelit,
   }
 
   ## Upper Limit of selected individuals is 100.
-  if (nPar > 100) {nPar <- 100}
+  max_selec <- getOption("windfarmGA.max_selection")
+  if (nPar > max_selec) {nPar <- max_selec}
 
   ## Randomly sample some individuals, based on their fitness value
   childsRunID <- sample(new1[, 1], nPar, prob = new1[, 'Parkfitness'], 
