@@ -13,7 +13,15 @@ utils::globalVariables(
 
 ## Used only or random_search_single for testing user-input
 .onLoad <- function(libname, pkgname){
-  options(windfarmGA.connection = stdin())
+  options(windfarmGA.connection = stdin(),
+          windfarmGA.cT = 0.88,
+          windfarmGA.air_rh = 1.225,
+          windfarmGA.k = 0.075,
+          windfarmGA.max_angle = 20,
+          windfarmGA.max_distance = 100000,
+          windfarmGA.max_population = 300,
+          windfarmGA.max_selection = 100
+          )
 }
 
 ## Is this still necessary?
