@@ -1,5 +1,6 @@
 library(testthat)
 library(windfarmGA)
-
+library(rgdal)
+library(rgeos)
 
 test_check("windfarmGA")
